EVENT_HANDLERS = {}

def event_handler(event_type: str):
    """
    A decorator that registers a function as a handler for a specific event_type.
    """
    def decorator(func):
        EVENT_HANDLERS[event_type] = func
        return func

    return decorator


import os

import django
from celery import Celery
from dotenv import load_dotenv

from eventbus.routes import ROUTES, QUEUES

load_dotenv()

DJANGO_PROJECT_NAME = os.environ['DJANGO_PROJECT_NAME']

RABBIT_HOST = os.environ["AWS_MQ_RABBITMQ_URL"]
RABBIT_PORT = os.getenv("AWS_MQ_RABBITMQ_PORT", "5671")
RABBIT_USER = os.getenv("AWS_MQ_RABBITMQ_USER", "guest")
RABBIT_PASS = os.getenv("AWS_MQ_RABBITMQ_PASSWORD", "guest")
BROKER_URL = f'amqps://{RABBIT_USER}:{RABBIT_PASS}@{RABBIT_HOST}:{RABBIT_PORT}'

os.environ.setdefault("DJANGO_SETTINGS_MODULE", f"{DJANGO_PROJECT_NAME}.settings")
django.setup()

app = Celery('eventbus', include=['eventbus.handlers'])
app.conf.update({
    'broker_url': BROKER_URL,
    'task_routes': ROUTES,
    'task_queues': QUEUES,
})

app.autodiscover_tasks(related_name='events')


from eventbus.worker import app


class EventClient:
    """
    A client for sending events between services.
    """

    @staticmethod
    def send_broadcast(event_type: str, payload: dict):
        """
        Send a broadcast event to all services.
        """
        EventClient._validate_event(event_type, payload)

        app.send_task(
            "events.broadcast",
            kwargs={"event_type": event_type, "payload": payload}
        )

    @staticmethod
    def send_direct(service_name: str, event_type: str, payload: dict):
        """
        Send a direct event to a specific service.
        """
        if not service_name or not isinstance(service_name, str):
            raise ValueError("`service_name` must be a non-empty string")

        EventClient._validate_event(event_type, payload)

        app.send_task(
            f"events.{service_name}",
            kwargs={"event_type": event_type, "payload": payload},
            routing_key=service_name
        )

    @staticmethod
    def _validate_event(event_type: str, payload: dict):
        """
        Validates event type and payload.
        """
        if not event_type or not isinstance(event_type, str):
            raise ValueError("`event_type` must be a non-empty string")

        if not isinstance(payload, dict):
            raise TypeError("`payload` must be a dictionary")

        if not payload:
            raise ValueError("`payload` cannot be empty")


import logging
import os

from eventbus.worker import app
from eventbus.registry import EVENT_HANDLERS

SERVICE_NAME = os.environ["SERVICE_NAME"]


@app.task(name="events.broadcast", broadcast=True)
def broadcast_event(event_type: str, payload: dict):
    """ Processing of broadcast events. """
    handler = EVENT_HANDLERS.get(event_type)

    if not handler:
        logging.info(f"[EVENTBUS] No handler for event_type = '{event_type}'")
        return

    logging.info(f"[EVENTBUS] Got event_type='{event_type}' | data={payload}")
    handler(**payload)


@app.task(name=f"events.{SERVICE_NAME}")
def direct_event(event_type: str, payload: dict):
    """ Processing of direct events. """
    handler = EVENT_HANDLERS.get(event_type)

    if not handler:
        logging.info(f"[EVENTBUS-DIRECT] No handler for event_type = '{event_type}'")
        return

    logging.info(f"[EVENTBUS-DIRECT] Got event_type='{event_type}' | data={payload}")
    handler(**payload)



import os

from dotenv import load_dotenv
from kombu import Queue, Exchange

load_dotenv()


SERVICE_NAME = os.environ["SERVICE_NAME"].strip()

DIRECT_ROUTES = {
    f'events.{SERVICE_NAME}': {
        'queue': f'{SERVICE_NAME}_queue',
        'exchange': 'services_exchange',
        'exchange_type': 'direct',
        'routing_key': SERVICE_NAME
    }
}
DIRECT_QUEUE = [
    Queue(
        name=f'{SERVICE_NAME}_queue',
        exchange=Exchange('services_exchange', type='direct', durable=True),
        routing_key=SERVICE_NAME
    )
]

BROADCAST_ROUTE = {
    'events.broadcast': {
        'exchange': 'broadcast_events',
        'exchange_type': 'fanout',
        'routing_key': '',
    }
}
BROADCAST_QUEUE = [
    Queue(
        name=f'{SERVICE_NAME}_broadcast',
        exchange=Exchange('broadcast_events', type='fanout', durable=True),
        routing_key=''
    )
]

ROUTES = {**BROADCAST_ROUTE, **DIRECT_ROUTES}
QUEUES = DIRECT_QUEUE + BROADCAST_QUEUE
